



pylint --help
IF %ERRORLEVEL% NEQ 0 exit 1
where pylint-gui
IF %ERRORLEVEL% NEQ 0 exit 1
where epylint
IF %ERRORLEVEL% NEQ 0 exit 1
pyreverse --help
IF %ERRORLEVEL% NEQ 0 exit 1
symilar --help
IF %ERRORLEVEL% NEQ 0 exit 1
exit 0
